@@ -3,7 +3,7 @@ package Class::DBI::ViewLoader;
 use strict;
 use warnings;
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 =head1 NAME
 
@@ -505,7 +505,7 @@ This is in addition to the class specified by the driver's base_class method,
 which will always be the first item in the generated @ISA. 
 
 Note that these classes are not loaded for you, be sure to C<use> or C<require>
-them manually before calling load_views().
+them yourself.
 
 =cut
 
@@ -558,7 +558,7 @@ the generated class itself.
     $loader->set_import_classes(qw(Class::DBI::Plugin::RetrieveAll));
 
 Any classes that inherit from Exporter will be loaded via Exporter's export()
-function. Any other classes are loaded by an import() in a string eval.
+function. Any other classes are loaded by an import() call in a string eval.
 
 =cut
 
@@ -604,15 +604,22 @@ sub get_import_classes { @{$_[0]->{_import_classes} || []} }
 The main method for the class, loads all relevant views from the database and
 generates classes for those views.
 
-The generated classes will and be read-only and have a multi-column primary key
-containing every column. This is because it is unlikely that the view will have
-a real primary key and Class::DBI insists that there should be a unique
+The generated classes will be read-only and have a multi-column primary key
+containing every column. This is because it is not guaranteed that the view will
+have a real primary key and Class::DBI insists that there should be a unique
 identifier for every row.
 
-Each class is only ever generated once, no matter how many times load_views() is
-called.
+Usually, any row containing an undef (NULL) primary key column is considered
+false in boolean context, in this particular case however that doesn't make much
+sense. So only all-null rows are considered false in classes generated by this
+class.
 
-Returns class names for all created classes, including any that already existed.
+Each class is only ever generated once, no matter how many times load_views() is
+called. If you want to load the same view twice for some reason, you can achieve
+this by changing the namespace.
+
+Returns class names for all created classes, including any that were skipped due
+to already existing.
 
 =cut
 
@@ -634,6 +641,9 @@ sub load_views {
 	}
     }
 
+    # load all symbols into all classes in a single call.
+    $self->_do_eval;
+
     return @classes;
 }
 
@@ -648,8 +658,18 @@ sub _create_class {
 
     {
 	no strict 'refs';
-	my $base = $self->base_class;
-	@{$class.'::ISA'} = ($base, $self->get_base_classes);
+
+	@{$class.'::ISA'} = ($self->base_class, $self->get_base_classes);
+
+	# We only want all-null primary keys to be considered false.
+	# (This method is used by the bool overloader)
+	*{$class.'::_undefined_primary'} = sub {
+	    my $self = shift;
+	    my @cols = $self->_attrs($self->primary_columns);
+	    my @undef = grep { not defined } @cols;
+
+	    return @undef == @cols ? 1 : 0;
+	};
     }
 
     $class->set_db(Main => $self->_get_dbi_args);
@@ -668,6 +688,9 @@ sub _create_class {
     return $class;
 }
 
+# import symbols into the target namespace. Try to avoid string eval when
+# possible. This eval code is cached by _set_eval and can be executed with
+# _do_eval
 sub _do_imports {
     my($self, $class) = @_;
 
@@ -690,8 +713,34 @@ sub _do_imports {
 
     if (@manual) {
 	# load classes via string eval (yuk!)
-	eval "package $class;\n\n".
-	    join("\n", map {"$_->import();"} @manual);
+	$self->_set_eval($class, @manual);
+    }
+
+    return $self;
+}
+
+# cache code to eval to minimise string eval calls
+sub _set_eval {
+    my ($self, $class, @manual) = @_;
+
+    my $code = join("\n",
+        "package $class;",
+        map {"$_->import();"} @manual
+    );
+
+    push @{$self->{__eval_string}}, $code;
+}
+
+# process pending eval code and reset
+sub _do_eval {
+    my $self = shift;
+
+    my $code = delete $self->{__eval_string};
+
+    if (defined $code) {
+        # trap errors?
+
+        eval join("\n\n", @$code);
     }
 
     return $self;
@@ -755,7 +804,7 @@ connection, like statement handles for example.
 	$self->SUPER::_clear_dbi_handle(@_);
     }
 
-=head1 DRIVERS
+=head1 DRIVER METHODS
 
 The following methods are provided by the relevant driver classes. If they are
 called on a native Class::DBI::ViewLoader object (one without a dsn set), they
@@ -904,3 +953,4 @@ the same terms as Perl itself.
 
 =cut
 
+vim: ts=8 sts=4 sw=4 noet sr
